package jp.co.sds_corp.util.daemon;

import java.util.List;

import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* 子サービスを順番に実行します。
* 途中のサービスが正常終了しなかった場合には、以降のサービスは実行されません。
*/
public class SequentialService extends NestedService
		implements SequentialServiceMBean
{
	static final CmnLogger logger = CmnLogFactory.getLogger();

	/**
	* 子サービスの実行を開始します。
	* 途中のサービスが正常終了しなかった場合には、以降の処理は行なわれません。
	*/
	public void start() {
		List<IService>	vecSvc = getServices();
		for ( int i = 0; i < vecSvc.size(); i++ ) {
			IService svc = (IService )vecSvc.get( i );
			if ( isDebug()) {
				logger.debug( this, "start service:" + svc );
			}
			svc.start();
			if ( svc.getStatus() != Status.WAIT ) {
				logger.error( this, "service status error:" + svc );
				break;
			}
			if ( isDebug()) {
				logger.debug( this, "end service:" + svc );
			}
		}
	}

	/**
	* 全てのサービスを停止します。
	*/
	public void stop() {
		List<IService>	vecSvc = getServices();
		for ( int i = 0; i < vecSvc.size(); i++ ) {
			IService svc = (IService )vecSvc.get( i );
			svc.stop();
		}
	}
}
