package	jp.co.sds_corp.util.daemon.remote;

import jp.co.sds_corp.util.config.DOMConfiguration;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.config.UnspecifiedPropertyException;

/**
* デーモンリモート制御するツールです。
* DaemonManagerが使用するXMLファイルから、リモート制御の情報を取得し
* DaemonManagerにコマンドを送信します。
*/
public class DaemonController
{
	IRemoteClient client;
	String	confFile = "daemon.xml";

	public DaemonController() throws Exception {
		if ( System.getProperty( "DaemonManager.config" ) != null ) {
			confFile = System.getProperty( "DaemonManager.config" );
		}
		IConfiguration config = new DOMConfiguration( confFile );
		init( config );
	}

	/**
	* デーモンマネージャの RMI I/Fを初期化します。
	* @param    config  設定情報
	*/
	protected void init( IConfiguration config ) throws Exception {
		IConfiguration regConf = config.getChild( "Remote" );
		if ( regConf == null ) {
			// @@@ 互換性のため
			regConf = config.getChild( "Registry" );
			if ( regConf != null ) {
				regConf.setAttribute( "type", "RMI" );
			}
		}
		if ( regConf == null ) {
			throw( new UnspecifiedPropertyException( "Remote" ));
		}
		client = RemoteFactory.createClient( regConf );
	}

	protected void start() throws Exception {
		client.connect();
	}

	protected void exec( String[] args ) throws Exception {
		Object ret = null;
		switch ( args.length ) {
		case 1:
			ret = client.execRemote( args[0] );
			break;
		default:
			String cmd = args[0];
			Class<?>[] cls = new Class<?>[args.length-1];
			Object[] prm = new Object[args.length-1];
			for ( int i = 1; i < args.length; i++ ) {
				cls[i-1] = args[i].getClass();
				prm[i-1] = args[i];
			}
			ret = client.execRemote( cmd, cls, prm );
			break;
		}
		if ( ret != null ) {
			if ( ret instanceof Throwable ) {
				((Throwable )ret).printStackTrace();
			} else {
				System.out.println( ret );
			}
		}
	}

	public IRemoteClient getClient() {
		return client;
	}

	private static void usage() {
		System.out.println(
			"usage: java DaemonController " +
				"action [param]");
		System.exit(0);
	}

	public static void main( String[] args ) {
		try {
			DaemonController ctl = new DaemonController();
			ctl.start();
			ctl.exec( args );
		} catch ( Exception ex ) {
			ex.printStackTrace();
		}
	}
}

